import { __rest } from 'tslib';
import * as i0 from '@angular/core';
import { Directive, Optional, Self, Input, HostListener, Injectable, NgModule } from '@angular/core';
import { fromEvent, merge } from 'rxjs';
import _Inputmask from 'inputmask';
import * as i1 from '@angular/cdk/platform';
import * as i2 from '@angular/forms';
import { formatDate, getCurrencySymbol, getLocaleNumberSymbol, NumberSymbol, getNumberOfCurrencyDigits } from '@angular/common';
import * as i1$1 from '@bravobit/bb-foundation/localize';

const createMask = (options) => typeof options === 'string' ? { mask: options } : options;

const InputmaskConstructor = _Inputmask.default || _Inputmask;
class BbInputMask {
    constructor(_ngZone, _platform, _renderer, _elementRef, _control) {
        this._ngZone = _ngZone;
        this._platform = _platform;
        this._renderer = _renderer;
        this._elementRef = _elementRef;
        this._control = _control;
        // State.
        this.inputMaskPlugin = null;
        this.nativeInputElement = null;
        this.eventsSubscription = null;
        this._inputMaskOptions = null;
        this._onChange = () => ({});
        this.onInput = (_) => ({});
        this.onTouched = () => ({});
        this.validate = (control) => {
            var _a;
            return !control.value || !this.inputMaskPlugin || ((_a = this.inputMaskPlugin) === null || _a === void 0 ? void 0 : _a.isValid())
                ? null
                : { inputMask: true };
        };
        this.nativeInputElement = this._elementRef.nativeElement;
        if (this._control != null) {
            this._control.valueAccessor = this;
        }
    }
    set bbInputMask(mask) {
        if (!mask) {
            return;
        }
        this._inputMaskOptions = mask;
        this.updateMask();
    }
    ngOnInit() {
        const control = this._control.control;
        if (!control) {
            return;
        }
        control.setValidators(control.validator ? [control.validator, this.validate] : [this.validate]);
        control.updateValueAndValidity();
    }
    ngOnDestroy() {
        this.destroyMask();
    }
    writeValue(value) {
        if (!this.nativeInputElement) {
            return;
        }
        this._renderer.setProperty(this.nativeInputElement, 'value', value !== null && value !== void 0 ? value : '');
    }
    registerOnChange(fn) {
        var _a;
        this._onChange = fn;
        const parser = (_a = this._inputMaskOptions) === null || _a === void 0 ? void 0 : _a.parser;
        this.onInput = value => this._onChange(parser ? parser(value) : value);
    }
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    setDisabledState(isDisabled) {
        if (!this.nativeInputElement) {
            return;
        }
        this._renderer.setProperty(this.nativeInputElement, 'disabled', isDisabled);
    }
    updateMask() {
        this.destroyMask();
        this.createMask();
        this.registerOnChange(this._onChange);
    }
    createMask() {
        var _a;
        if (!this._platform.isBrowser || !this.nativeInputElement || this._inputMaskOptions === null || Object.keys(this._inputMaskOptions).length === 0) {
            return;
        }
        const _b = this._inputMaskOptions, { parser } = _b, options = __rest(_b, ["parser"]);
        this.inputMaskPlugin = this._ngZone.runOutsideAngular(() => new InputmaskConstructor(options).mask(this.nativeInputElement));
        const setValue$ = fromEvent(this.nativeInputElement, 'setvalue');
        const cleared$ = fromEvent(this.nativeInputElement, 'cleared');
        this.eventsSubscription = merge(setValue$, cleared$)
            .subscribe(event => { var _a; return this.onInput((_a = event === null || event === void 0 ? void 0 : event.target) === null || _a === void 0 ? void 0 : _a['value']); });
        const control = (_a = this._control) === null || _a === void 0 ? void 0 : _a.control;
        if (!control) {
            return;
        }
        setTimeout(() => control === null || control === void 0 ? void 0 : control.updateValueAndValidity());
    }
    destroyMask() {
        var _a, _b;
        (_a = this.eventsSubscription) === null || _a === void 0 ? void 0 : _a.unsubscribe();
        this.eventsSubscription = null;
        (_b = this.inputMaskPlugin) === null || _b === void 0 ? void 0 : _b.remove();
        this.inputMaskPlugin = null;
    }
}
BbInputMask.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.0.4", ngImport: i0, type: BbInputMask, deps: [{ token: i0.NgZone }, { token: i1.Platform }, { token: i0.Renderer2 }, { token: i0.ElementRef }, { token: i2.NgControl, optional: true, self: true }], target: i0.ɵɵFactoryTarget.Directive });
BbInputMask.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "14.0.4", type: BbInputMask, selector: "input[bbInputMask]", inputs: { bbInputMask: "bbInputMask" }, host: { listeners: { "input": "onInput($event.target.value)", "blur": "onTouched($event.target.value)" } }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.0.4", ngImport: i0, type: BbInputMask, decorators: [{
            type: Directive,
            args: [{
                    selector: 'input[bbInputMask]'
                }]
        }], ctorParameters: function () {
        return [{ type: i0.NgZone }, { type: i1.Platform }, { type: i0.Renderer2 }, { type: i0.ElementRef }, { type: i2.NgControl, decorators: [{
                        type: Optional
                    }, {
                        type: Self
                    }] }];
    }, propDecorators: { bbInputMask: [{
                type: Input
            }], onInput: [{
                type: HostListener,
                args: ['input', ['$event.target.value']]
            }], onTouched: [{
                type: HostListener,
                args: ['blur', ['$event.target.value']]
            }] } });

class Masking {
    constructor(_localize) {
        this._localize = _localize;
    }
    date(format = 2 /* DateMaskFormat.YearMonthDay */, separator = '-') {
        const dateParts = this.getDateParts(format);
        const inputFormat = dateParts.join(separator);
        const outputFormat = 'yyyy-MM-dd';
        const escape = /[.*+?^${}()\/|[\]\\]/g;
        const escapedSeparator = separator.replace(escape, '\\$&');
        const inputFormatRegexString = dateParts
            .map(item => `(?<${item}>[0-9]{${item === null || item === void 0 ? void 0 : item.length}})`)
            .join(escapedSeparator);
        const inputFormatRegex = new RegExp(`^${inputFormatRegexString}$`);
        return createMask({
            alias: 'datetime',
            inputFormat: inputFormat.toLowerCase(),
            outputFormat: outputFormat.toLowerCase(),
            nullable: false,
            inputmode: 'numeric',
            onBeforeMask: (initialValue) => {
                return initialValue ? formatDate(initialValue, inputFormat, 'en-US') : null;
            },
            onBeforePaste: (value) => {
                return value;
            },
            parser: (value) => {
                var _a, _b, _c;
                if (!value || typeof value !== 'string' || !inputFormatRegex.test(value)) {
                    return null;
                }
                const matches = value.match(inputFormatRegex);
                const year = +((_a = matches === null || matches === void 0 ? void 0 : matches.groups) === null || _a === void 0 ? void 0 : _a['yyyy']);
                const month = +((_b = matches === null || matches === void 0 ? void 0 : matches.groups) === null || _b === void 0 ? void 0 : _b['MM']) - 1;
                const day = +((_c = matches === null || matches === void 0 ? void 0 : matches.groups) === null || _c === void 0 ? void 0 : _c['dd']);
                const date = new Date(year, month, day);
                return date instanceof Date && !isNaN(date)
                    ? formatDate(date, outputFormat, 'en-US')
                    : null;
            }
        });
    }
    currency(code, options) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p;
        const locale = (_c = (_b = (_a = this._localize) === null || _a === void 0 ? void 0 : _a.current) === null || _b === void 0 ? void 0 : _b.locale) !== null && _c !== void 0 ? _c : 'en';
        const symbol = (_d = options === null || options === void 0 ? void 0 : options.symbol) !== null && _d !== void 0 ? _d : getCurrencySymbol(code, (_e = options === null || options === void 0 ? void 0 : options.symbolFormat) !== null && _e !== void 0 ? _e : 'narrow', locale);
        const decimal = (_f = options === null || options === void 0 ? void 0 : options.decimalCharacter) !== null && _f !== void 0 ? _f : getLocaleNumberSymbol(locale, NumberSymbol.CurrencyDecimal);
        const group = (_g = options === null || options === void 0 ? void 0 : options.groupCharacter) !== null && _g !== void 0 ? _g : getLocaleNumberSymbol(locale, NumberSymbol.CurrencyGroup);
        const digits = (_h = options === null || options === void 0 ? void 0 : options.digits) !== null && _h !== void 0 ? _h : getNumberOfCurrencyDigits(code);
        const spacer = (_j = options === null || options === void 0 ? void 0 : options.spacer) !== null && _j !== void 0 ? _j : ' ';
        return createMask({
            alias: 'numeric',
            digits: digits,
            allowMinus: (_k = options === null || options === void 0 ? void 0 : options.allowMinus) !== null && _k !== void 0 ? _k : false,
            min: (_l = options === null || options === void 0 ? void 0 : options.min) !== null && _l !== void 0 ? _l : 0,
            max: (_m = options === null || options === void 0 ? void 0 : options.max) !== null && _m !== void 0 ? _m : Number.MAX_SAFE_INTEGER,
            prefix: `${symbol}${spacer}`,
            radixPoint: decimal !== null && decimal !== void 0 ? decimal : '.',
            groupSeparator: group !== null && group !== void 0 ? group : ',',
            digitsOptional: (_o = options === null || options === void 0 ? void 0 : options.digitsOptional) !== null && _o !== void 0 ? _o : false,
            nullable: (_p = options === null || options === void 0 ? void 0 : options.nullable) !== null && _p !== void 0 ? _p : true,
            autoUnmask: true,
            unmaskAsNumber: true,
            rightAlign: false,
            showMaskOnHover: false,
            showMaskOnFocus: false,
            inputmode: digits > 0 ? 'decimal' : 'numeric',
            parser: (value) => {
                if (value === '') {
                    return null;
                }
                return value;
            }
        });
    }
    getDateParts(format) {
        switch (format) {
            case 0 /* DateMaskFormat.DayMonthYear */:
                return ['dd', 'MM', 'yyyy'];
            case 1 /* DateMaskFormat.MonthDayYear */:
                return ['MM', 'dd', 'yyyy'];
            case 3 /* DateMaskFormat.YearDayMonth */:
                return ['yyyy', 'dd', 'MM'];
            case 2 /* DateMaskFormat.YearMonthDay */:
            default:
                return ['yyyy', 'MM', 'dd'];
        }
    }
}
Masking.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.0.4", ngImport: i0, type: Masking, deps: [{ token: i1$1.Localize, optional: true }], target: i0.ɵɵFactoryTarget.Injectable });
Masking.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "14.0.4", ngImport: i0, type: Masking, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.0.4", ngImport: i0, type: Masking, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () {
        return [{ type: i1$1.Localize, decorators: [{
                        type: Optional
                    }] }];
    } });

class MaskingModule {
}
MaskingModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.0.4", ngImport: i0, type: MaskingModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
MaskingModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "14.0.4", ngImport: i0, type: MaskingModule, declarations: [BbInputMask], exports: [BbInputMask] });
MaskingModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "14.0.4", ngImport: i0, type: MaskingModule });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.0.4", ngImport: i0, type: MaskingModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [BbInputMask],
                    exports: [BbInputMask]
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { BbInputMask, Masking, MaskingModule, createMask };
//# sourceMappingURL=bravobit-bb-foundation-masking.mjs.map
